/*
 * (C) Copyright 2011 Samsung Electronics Co. Ltd
 *
 * See file CREDITS for list of people who contributed to this
 * project.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 */

#include <common.h>
#include <asm/arch/movi_partition.h>
#include <asm/arch/cpu.h>

#ifndef CONFIG_SPL_BUILD
#ifdef DEBUG_MOVI_PARTITION
#define dbg(x...)       printf(x)
#else
#define dbg(x...)       do { } while (0)
#endif


raw_area_t raw_area_control;

int init_raw_area_table(int location)
{
	int i;
	int partition_index = 0;
	member_t *image;

	/* init raw_area will be 16MB */
	raw_area_control.start_blk = 16*1024*1024/MOVI_BLKSIZE;
	raw_area_control.next_raw_area = 0;
	strcpy(raw_area_control.description, "init raw table");

	image = raw_area_control.image;

	/* For eMMC partition BLOCK Change*/

	/* image 0 should be bl1 */
	image[partition_index].start_blk = (location == 1) ? 1 : 0;
	image[partition_index].used_blk = MOVI_BL1_BLKCNT;
	image[partition_index].size = PART_SIZE_BL1;
	image[partition_index].attribute = partition_index;
	strcpy(image[partition_index].description, "fwbl1");
	dbg("fwbl1: %d\n", image[partition_index].start_blk);
	partition_index++;

	/* image 1 should be secure monitor */
	image[partition_index].start_blk = image[partition_index - 1].start_blk + MOVI_BL1_BLKCNT;
	image[partition_index].used_blk = MOVI_EL3_MON_BLKCNT;
	image[partition_index].size = PART_SIZE_EL3_MON;
	image[partition_index].attribute = partition_index;
	strcpy(image[partition_index].description, "el3_mon");
	dbg("el3_mon: %d\n", image[partition_index].start_blk);
	partition_index++;

	/* image 2 should be bl2 */
	image[partition_index].start_blk = image[partition_index - 1].start_blk + MOVI_EL3_MON_BLKCNT;
#if defined(CONFIG_ECT)
#ifdef CONFIG_SCSI_UFS
	image[partition_index].used_blk = MOVI_BL2_BLKCNT + MOVI_ECT_BLKCNT;
#else
	image[partition_index].used_blk = MOVI_BL2_BLKCNT + MOVI_ECT_BLKCNT + MOVI_ECT_BLKCNT;
#endif
	image[partition_index].size = PART_SIZE_BL2 + PART_SIZE_ECT;
#else
	image[partition_index].used_blk = MOVI_BL2_BLKCNT;
	image[partition_index].size = PART_SIZE_BL2;
#endif
	image[partition_index].attribute = partition_index;
	strcpy(image[partition_index].description, "bl2");
	dbg("bl2: %d\n", image[partition_index].start_blk);
	partition_index++;

#if defined(CONFIG_ECT)
	/* image 3 should be ect_default */
	image[partition_index].start_blk = image[partition_index - 1].start_blk + MOVI_BL2_BLKCNT - MOVI_BL2_SIG_BLKCNT;
	image[partition_index].used_blk = MOVI_ECT_BLKCNT;
	image[partition_index].size = PART_SIZE_ECT;
	image[partition_index].attribute = partition_index;
	strcpy(image[partition_index].description, "ect_default");
	dbg("ect_default: %d\n", image[partition_index].start_blk);
	partition_index++;

	/* image 4 should be ect_test */
	image[partition_index].start_blk = image[partition_index - 1].start_blk + MOVI_BL2_SIG_BLKCNT + MOVI_ECT_BLKCNT;
	image[partition_index].used_blk = MOVI_ECT_BLKCNT;
	image[partition_index].size = PART_SIZE_ECT;
	image[partition_index].attribute = partition_index;
	strcpy(image[partition_index].description, "ect_test");
	dbg("ect_test: %d\n", image[partition_index].start_blk);
	partition_index++;

	/* image 5 should be u-boot */
	image[partition_index].start_blk = image[partition_index - 1].start_blk + MOVI_ECT_BLKCNT;
	image[partition_index].used_blk = MOVI_UBOOT_BLKCNT;
	image[partition_index].size = PART_SIZE_UBOOT;
	image[partition_index].attribute = partition_index;
	strcpy(image[partition_index].description, "u-boot");
	dbg("u-boot: %d\n", image[partition_index].start_blk);
	partition_index++;
#else
	/* image should be u-boot */
	image[partition_index].start_blk = image[partition_index - 1].start_blk + MOVI_BL2_BLKCNT;
	image[partition_index].used_blk = MOVI_UBOOT_BLKCNT;
	image[partition_index].size = PART_SIZE_UBOOT;
	image[partition_index].attribute = partition_index;
	strcpy(image[partition_index].description, "u-boot");
	dbg("u-boot: %d\n", image[partition_index].start_blk);
	partition_index++;
#endif
	/* image 6 should be TrustZone S/W */
	image[partition_index].start_blk = image[partition_index - 1].start_blk + MOVI_UBOOT_BLKCNT;
	image[partition_index].used_blk = MOVI_MOBICORE_BLKCNT;
	image[partition_index].size = PART_SIZE_MOBICORE;
	image[partition_index].attribute = partition_index;
	strcpy(image[partition_index].description, "TrustZone S/W");
	dbg("TrustZone S/W: %d\n", image[partition_index].start_blk);
	partition_index++;

	/* image 7 should be clogo */
	image[partition_index].start_blk = MOVI_CLOGO_POS;
	image[partition_index].used_blk = MOVI_CLOGO_BLKCNT;
	image[partition_index].size = PART_SIZE_CLOGO;
	image[partition_index].attribute = partition_index;
	strcpy(image[partition_index].description, "clogo");
	dbg("clogo: %d\n", image[partition_index].start_blk);
	partition_index++;

	/* image 8 should be Loadable F/W */
	image[partition_index].start_blk = image[partition_index - 1].start_blk + MOVI_CLOGO_BLKCNT;
	image[partition_index].used_blk = MOVI_LOADABLE_FW_BLKCNT;
	image[partition_index].size = PART_SIZE_LOADABLE_FW;
	image[partition_index].attribute = partition_index;
	strcpy(image[partition_index].description, "ldfw");
	dbg("Loadable F/W: %d\n", image[partition_index].start_blk);
	partition_index++;


	for (i = partition_index; i< 15; i++) {
		raw_area_control.image[i].start_blk = 0;
		raw_area_control.image[i].used_blk = 0;
	}

	return 0;
}
#endif /* CONFIG_SPL_BUILD */
