/*
 * Copyright@ Samsung Electronics Co. LTD
 *
 * This software is proprietary of Samsung Electronics.
 * No part of this software, either material or conceptual may be copied or distributed, transmitted,
 * transcribed, stored in a retrieval system or translated into any human or computer language in any form by any means,
 * electronic, mechanical, manual or otherwise, or disclosed
 * to third parties without the express written permission of Samsung Electronics.


 * Alternatively, this program is free software in case of open source project
 * you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.

*/

#include "asv.h"
#include "bl2_smc.h"
#include "bl2_io.h"
#include "smc_aarch64.h"

#ifndef CONFIG_SG_BINARY

#define CHIPID_BASE	(void *)(0x10100000)
#define OTP_CON_TOP_BASE	(void *)(0x100D0000)

#define CMU_RESERVED	0x02053000
static unsigned char *cp_cpu_freq = (unsigned char *)(CMU_RESERVED + 0x61);

struct chip_info chip_info;

void init_chip_info(void)
{
	unsigned int uBits;

	/* ASV Group */
	uBits = (unsigned int)exynos_smc_read(SMC_CMD_REG, SMC_REG_ID_SFR_R((unsigned long long)(OTP_CON_TOP_BASE + 0x9000)));
	chip_info.nAsv_CPUCL0 = ((unsigned char)((char)((uBits >> 0) & 0xF) + (char)((uBits >> 4) & 0xF))) & 0xF;

	uBits = (unsigned int)exynos_smc_read(SMC_CMD_REG, SMC_REG_ID_SFR_R((unsigned long long)(OTP_CON_TOP_BASE + 0x9004)));
	chip_info.nAsv_G3D = ((unsigned char)((char)((uBits >> 0) & 0xF) + (char)((uBits >> 4) & 0xF))) & 0xF;
	chip_info.nAsv_MIF = ((unsigned char)((char)((uBits >> 16) & 0xF) + (char)((uBits >> 20) & 0xF))) & 0xF;

	uBits = (unsigned int)exynos_smc_read(SMC_CMD_REG, SMC_REG_ID_SFR_R((unsigned long long)(OTP_CON_TOP_BASE + 0x9008)));
	chip_info.nAsv_INT = ((unsigned char)((char)((uBits >> 0) & 0xF) + (char)((uBits >> 4) & 0xF))) & 0xF;
	chip_info.nAsv_CAM_DISP = ((unsigned char)((char)((uBits >> 16) & 0xF) + (char)((uBits >> 20) & 0xF))) & 0xF;

	uBits = (unsigned int)exynos_smc_read(SMC_CMD_REG, SMC_REG_ID_SFR_R((unsigned long long)(OTP_CON_TOP_BASE + 0x900C)));
	chip_info.nAsv_TABLE = ((unsigned char)((uBits >> 0) & 0x7F));
	chip_info.nAsv_CP = ((unsigned char)((char)((uBits >> 16) & 0xF) + (char)((uBits >> 20) & 0xF))) & 0xF;

	uBits = (unsigned int)exynos_smc_read(SMC_CMD_REG, SMC_REG_ID_SFR_R((unsigned long long)(OTP_CON_TOP_BASE + 0x903C)));
	chip_info.nAsv_CP_CPU_Freq = ((unsigned char)((uBits >> 29) & 0x7));
	*cp_cpu_freq = chip_info.nAsv_CP_CPU_Freq;
}
#endif
