/*
 * Copyright@ Samsung Electronics Co. LTD
 *
 * This software is proprietary of Samsung Electronics.
 * No part of this software, either material or conceptual may be copied or distributed, transmitted,
 * transcribed, stored in a retrieval system or translated into any human or computer language in any form by any means,
 * electronic, mechanical, manual or otherwise, or disclosed
 * to third parties without the express written permission of Samsung Electronics.


 * Alternatively, this program is free software in case of open source projec;
 * you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.

*/

#include "bl2_config.h"
#include "smc_aarch64.h"

#define SIGNATURE_CHECK_FAIL	(-1)
#define SIGNATURE_CHECK_SUCCESS	(0)

typedef enum {
	BD_NO_DEVICE = 0x0,
	BD_UFS = 0x1,
	BD_EMMC_BOOT = 0x2,
	BD_ERROR = 0x3,
	BD_USB = 0x4,
	BD_SDMMC = 0x5,
	BD_UFS_CARD = 0x6,
	BD_SPI = 0x7,
} boot_device_info_t;

void bl2_load_binary(void)
{
	unsigned int order = 0;
	unsigned int boot_device = 0;
	unsigned int boot_device_info = 0;
	int ret = 0;

	boot_device_info = find_second_boot();
	order = boot_device_info & 0xF;
	switch ((boot_device_info >> (4 * order)) & 0xF) {
	case BD_UFS:
		boot_device = UFS;
		break;
	case BD_EMMC_BOOT:
		boot_device = EMMC;
		break;
	case BD_USB:
		boot_device = USB;
		break;
	case BD_SDMMC:
		boot_device = SDMMC_CH2;
		break;
	case BD_UFS_CARD:
		break;
	case BD_SPI:
		break;
	default:
		break;
	}

	/* Load u-boot image to ram */
	ret = load_bl3_image(boot_device);
	if (ret != SIGNATURE_CHECK_SUCCESS)
		while (1);

	/* Load tzsw image & U-Boot boot */
	ret = coldboot(boot_device);
	if (ret != SIGNATURE_CHECK_SUCCESS)
		while (1);
}
