/*
 * Copyright@ Samsung Electronics Co. LTD
 *
 * This software is proprietary of Samsung Electronics.
 * No part of this software, either material or conceptual may be copied or distributed, transmitted,
 * transcribed, stored in a retrieval system or translated into any human or computer language in any form by any means,
 * electronic, mechanical, manual or otherwise, or disclosed
 * to third parties without the express written permission of Samsung Electronics.


 * Alternatively, this program is free software in case of open source project
 * you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.

*/

#ifndef __S2MPW01_PMIC_H__
#define __S2MPW01_PMIC_H__

/* S2MPW01 slave address */
#define S2MPW01_PM_ADDR	0x1

/* S2MPW01 Register Address */
#define S2MPW01_PM_RTC_BUF	0x0B
#define S2MPW01_PM_CTRL1	0x0C
#define S2MPW01_PM_CTRL3	0x0E
#define S2MPW01_PM_TIME_CTRL1	0x11

#define S2MPW01_PM_BUCK1_OUT1	0x14
#define S2MPW01_PM_BUCK1_OUT2	0x15
#define S2MPW01_PM_BUCK1_OUT3	0x16
#define S2MPW01_PM_LDO1_CTRL1	0x20
#define S2MPW01_PM_LDO1_CTRL2	0x21

/*
 * LDO OUT mask [5:0]
 */
#define LDOOUT_MASK		0x3F

/*
 * RTC_BUF
 */
#define S2MPW01_PERI_32KHZ_EN	(0x1 << 2)
#define S2MPW01_AP_32KHZ_EN	(0x1 << 0)

/*
 * CTRL1
 */
#define S2MPW01_MRSTBEN	(0x1 << 4)

/*
 * CTRL3
 */
#define S2MPW01_WRSTEN		(0x1 << 4)

/*
 * TIME_CTRL1
 */
#define S2MPW01_WRSTBIEN	(0x1 << 1)

#define VDD_BASE_VOLT		50000
#define VDD_VOLT_STEP		625
#define MIN_VOLT		500
#define MAX_RD_VAL		0xB0
#define RD_BUCK_VOLT(x)	(((x > MAX_RD_VAL) ? 0 : \
				(((x - 0x10) * VDD_VOLT_STEP) + VDD_BASE_VOLT) / 100))
#define WR_BUCK_VOLT(x)	((x < MIN_VOLT) ? 0x10 : \
				((((x) * 100) - VDD_BASE_VOLT) / VDD_VOLT_STEP + 0x10))

void pmic_enable_peric_dev(void);
void pmic_gpio_init(void);

#endif /*__S2MPW01_PMIC_H__*/

