/*
 * Copyright@ Samsung Electronics Co. LTD
 *
 * This software is proprietary of Samsung Electronics.
 * No part of this software, either material or conceptual may be copied or distributed, transmitted,
 * transcribed, stored in a retrieval system or translated into any human or computer language in any form by any means,
 * electronic, mechanical, manual or otherwise, or disclosed
 * to third parties without the express written permission of Samsung Electronics.


 * Alternatively, this program is free software in case of open source projec;
 * you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.

*/

#ifndef __PRESET_QOS_H__
#define __PRESET_QOS_H__

#define PRESET_DECON_QOS		1

/* for setting DECON QOS value */
#if PRESET_DECON_QOS
#define EXYNOS7570_PA_DREX              0x10400000
#define QOS_TIMEOUT_0xA                 0x328
#define EXYNOS7570_PA_TREX_DISPAUD      0x12460000
#define TREX_CON                        0x000
#define TREX_RCON                       0x020
#define TREX_WCON                       0x040

static inline void trex_setqos(void *base, unsigned int priority)
{
	unsigned int tmp_reg = 0;

	/* override qos value */
	tmp_reg |= 1 << 8;
	tmp_reg |= (priority & 0xf) << 12;

	__raw_writel(tmp_reg, base + TREX_RCON);
	__raw_writel(tmp_reg, base + TREX_WCON);

	/* qos on */
	__raw_writel(0x1, base + TREX_CON);
}

static inline void decon_bts_conf(void)
{
	/* decon bts priority: 0xa, drex timeout: 0x40 */
	__raw_writel(0x40, EXYNOS7570_PA_DREX + QOS_TIMEOUT_0xA);
	trex_setqos((void *)EXYNOS7570_PA_TREX_DISPAUD, 0xa);
}
#else
static inline void trex_setqos(void *base, unsigned int priority) { return; }
static inline void decon_bts_conf(void) { return; }
#endif /* PRESET_DECON_QOS */
#endif /* __PRESET_QOS_H__ */
