/* board/samsung/espresso7570/display.c
 *
 * SAMSUNG Exynos display configuration file.
 *
 * Copyright@ Samsung Electronics Co. LTD
 *
 * Manseok Kim <manseoks.kim@samsung.com> *
 * Yeongran Shin <yr613.shin@samsung.com> *
 *
 * This software is proprietary of Samsung Electronics.
 * No part of this software, either material or conceptual may be copied or distributed, transmitted,
 * transcribed, stored in a retrieval system or translated into any human or computer language in any form by any means,
 * electronic, mechanical, manual or otherwise, or disclosed
 * to third parties without the express written permission of Samsung Electronics.


 * Alternatively, this program is free software in case of open source projec;
 * you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
*/


#include <common.h>
#include <lcd.h>
#include <asm/arch/gpio.h>
#include <asm/arch/exynos7570.h>
#include <asm/arch/power.h>
#include <asm/arch/sysreg.h>
#include <asm/arch/cpu.h>
#include <asm/arch-exynos/pmu.h>

#define MIPI_DPHY_M4S2_M_RESETN		(1 << 0)

/*
 * ########## Machine dependency ##########
*/
static void exynos7570_mipi_phy_control(unsigned int dev_index,
					unsigned int enable)
{
	unsigned int addr, cfg = 0;

	/* Enable DPHY reset(0) -> Release DPHY reset(1) */
	addr = EXYNOS7570_SYSREG_DISPAUD_MIPI_PHY_CON;
	cfg = readl(addr);

	cfg &= ~(MIPI_DPHY_M4S2_M_RESETN);
	writel(cfg, addr);
	cfg |= MIPI_DPHY_M4S2_M_RESETN;
	writel(cfg, addr);

	/* DPHY isolation with PMU */
	if (dev_index == 0)
		addr = EXYNOS7570_POWER_MIPI_PHY_M4S2_CONTROL;

	cfg = readl(addr);
	if (enable)
		cfg |= EXYNOS_MIPI_PHY_ENABLE;
	else
		cfg &= ~(EXYNOS_MIPI_PHY_ENABLE);

	writel(cfg, addr);
}


/* MIPI-PHY related with master interface */
static void set_mipi_phy_control(unsigned int enable)
{
	exynos7570_mipi_phy_control(0, enable);
}

/* Set GPIO internal interrupt when DECON are using HW trigger */
static void set_gpio_hw_te(void)
{
	/* It is not required at video mode */
}

/*
 * ########## Board dependency ##########
*/
/* Configure and set a GPIO for LCD_RESET */
static void set_gpio_lcd_reset(enum board_gpio_type gpio_type)
{
	struct s5p_gpio_bank *bank_0 = NULL;
	int gpio_0 = 0;

	switch (gpio_type) {
	case BOARD_BTYPE:
		/* GPIO: DISP_RST_N for universal */
		/* GPD2_CON, GPD2_0 [3:0] 0x1 = Output */
		bank_0 = (struct s5p_gpio_bank *)EXYNOS7570_GPD2CON;
		gpio_0 = 0;
		break;
	case BOARD_ESPRESSO:
		/* GPIO: LCD RESET */
		/* GPD2_CON, GPD2_0 [3:0] 0x1 = Output */
		bank_0 = (struct s5p_gpio_bank *)EXYNOS7570_GPD2CON;
		gpio_0 = 0;
		break;
	default:
		debug("%s: Does not support it!!\n", __func__);
		return;
	}
	s5p_gpio_set_pull(bank_0, gpio_0, GPIO_PULL_DOWN);
	s5p_gpio_cfg_pin(bank_0, gpio_0, GPIO_OUTPUT);
	s5p_gpio_set_drv(bank_0, gpio_0, GPIO_DRV_4X);

	/* RESET: "0" -> "1" */
	udelay(5000);
	s5p_gpio_set_value(bank_0, gpio_0, 0);
	udelay(5000);
	s5p_gpio_set_value(bank_0, gpio_0, 1);
	udelay(10000);

}

/* Configure and set a GPIO for LCD_POWER_ON */
static void set_gpio_lcd_power(enum board_gpio_type gpio_type)
{
	struct s5p_gpio_bank *bank_0 = NULL;
	int gpio_0 = 0;

	switch (gpio_type) {
	case BOARD_BTYPE:
		bank_0 = (struct s5p_gpio_bank *)EXYNOS7570_GPA0CON;
		gpio_0 = 1;
		break;
	case BOARD_ESPRESSO:
		bank_0 = (struct s5p_gpio_bank *)EXYNOS7570_GPA0CON;
		gpio_0 = 1;
		break;
	default:
		debug("%s: Does not support it!!\n", __func__);
		return;
	}
	s5p_gpio_set_pull(bank_0, gpio_0, GPIO_PULL_DOWN);
	s5p_gpio_cfg_pin(bank_0, gpio_0, GPIO_OUTPUT);
	s5p_gpio_set_drv(bank_0, gpio_0, GPIO_DRV_4X);

	s5p_gpio_set_value(bank_0, gpio_0, 1);
	udelay(10000);
}

struct exynos_display_config display_config = {
	.set_mipi_phy = set_mipi_phy_control,
	.set_gpio_hw_te = set_gpio_hw_te,
	.set_gpio_lcd_reset = set_gpio_lcd_reset,
	.set_gpio_lcd_power = set_gpio_lcd_power,
};

/* Get type of board */
enum board_gpio_type get_exynos_board_type(void)
{
#if defined (CONFIG_MACH_ESPRESSO7570)
	return BOARD_ESPRESSO;
#elif defined (CONFIG_MACH_BTYPE7570)
	return BOARD_BTYPE;
#else
	return BOARD_ESPRESSO;
#endif
}

/* Get display configuration callbacks */
struct exynos_display_config *get_exynos_display_config(void)
{
	return &display_config;
}
