/**
 * @file  otp.h
 * @brief defines for OTP control
 *
 * Copyright (c) 2016 Samsung Electronics Co., Ltd.
 *
 * This software is proprietary of Samsung Electronics.
 * No part of this software, either material or conceptual may be copied
 * or distributed, transmitted, transcribed, stored in a retrieval system
 * or translated into any human or computer language in any form by any means,
 * electronic, mechanical, manual or otherwise, or disclosed to third parties
 * without the express written permission of Samsung Electronics.
 *
 * Alternatively, this program is free software in case of open source project
 * you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 */

#ifndef __OTP_H__
#define __OTP_H__

/*****************************************************************************/
/* ERROR Codes                                                               */
/*****************************************************************************/
#define RV_SUCCESS			0

#define RV_OTP_LDFW_NOT_LOADED		0xFFFFFFFF

#define RV_OTP_OEM_KEY_CMP_FAIL		0x30256
#define RV_OTP_JTAG_KEY_CMP_FAIL	0x30257

#define RV_OTP_PARAM_NULL_POINTER	0x85000
#define RV_OTP_PARAM_INVALID_KEY_LEN	0x85001

/*****************************************************************************/
/* SMC ID                                                                    */
/*****************************************************************************/
/* SMC ID for OTP control */
#define SMC_AARCH64_PREFIX		0xC2000000
#define SMC_CM_OTP_CONTROL		0x1100

#define OTP_OEM_KEY_SIZE		32
#define OTP_JTAG_KEY_SIZE		64

/*****************************************************************************/
/* Commands                                                                  */
/*****************************************************************************/
/* command list */
#define CMD_W_OEM_KEY0_0		16
#define CMD_W_OEM_KEY0_1		17
#define CMD_W_OEM_KEY1_0		18
#define CMD_W_OEM_KEY1_1		19

#define CMD_R_USE_OEM_KEY		21

#define CMD_R_JTAG_LOCK			25

#define CMD_W_JTAG_KEY_0		50
#define CMD_W_JTAG_KEY_1		51
#define CMD_W_JTAG_KEY_2		52
#define CMD_W_JTAG_KEY_3		53
#define CMD_W_JTAG_KEY_4		54
#define CMD_W_JTAG_KEY_5		55
#define CMD_W_JTAG_KEY_6		56
#define CMD_W_JTAG_KEY_7		57

/*****************************************************************************/
/* API functions                                                             */
/*****************************************************************************/
uint64_t cm_otp_check_secure_boot_status(uint64_t *secure_boot_status);
uint64_t cm_otp_check_secure_jtag_status(uint64_t *secure_jtag_status);
uint64_t cm_otp_enable_secure_boot(uint8_t *oem_key_ptr, uint32_t oem_key_len);
uint64_t cm_otp_enable_secure_jtag(uint8_t *jtag_key_ptr, uint32_t jtag_key_len);

#endif /* __OTP_H__ */
