/*
 * Copyright (C) 2016 Samsung Electronics
 *
 * See file CREDITS for list of people who contributed to this
 * project.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 * MA 02111-1307 USA
 */

#include <asm/arch/smc.h>
#include <asm/arch/pmu.h>

u64 exynos_smc(u64 cmd, u64 arg1, u64 arg2, u64 arg3)
{
	register u64 reg0 __asm__("x0") = cmd;
	register u64 reg1 __asm__("x1") = arg1;
	register u64 reg2 __asm__("x2") = arg2;
	register u64 reg3 __asm__("x3") = arg3;

	__asm__ volatile (
		"smc	0\n"
		: "+r"(reg0), "+r"(reg1), "+r"(reg2), "+r"(reg3)

	);

	return reg0;
}

inline u64 exynos_smc_read(u64 cmd, u64 arg1)
{
	register u64 reg0 __asm__("x0") = cmd;
	register u64 reg1 __asm__("x1") = arg1;
	register u64 reg2 __asm__("x2") = 0;

	__asm__ volatile (
		"dsb	sy\n"
		"smc	0\n"
		: "+r"(reg0), "+r"(reg1), "+r"(reg2)

	);

	return reg2;
}

int init_ldfw(u64 addr, u64 size)
{
	return exynos_smc(SMC_CMD_LOAD_LDFW,
			addr, size, 0);
}

unsigned int find_second_boot(void)
{
	return exynos_smc_read(SMC_CMD_CHECK_SECOND_BOOT, 0);
}

void emmc_endbootop(void)
{
	exynos_smc(SMC_CMD_EMMC_ENDBOOTOP, 0, 0, 0);
}

void sdmmc_enumerate(void)
{
	exynos_smc(SMC_CMD_SDMMC_ENUMERATE, 0, 0, 0);
}

void set_secure_reg(u64 reg_val, u32 num)
{
	exynos_smc(SMC_CMD_SET_SECURE_REG, reg_val, (u64)num, reg_val);
}

#if defined(CONFIG_RAMDUMP_GPR)
void cpu_boot(u64 id, u64 cpu, u64 fn)
{
	exynos_smc(id, cpu, fn, 0);
}
#endif

unsigned int get_el3_mon_version(char *ptr, u32 string_size)
{
	return exynos_smc(SMC_CMD_GET_EL3_MON_VERSION,
			(u64)ptr, (u64)string_size, 0);
}

u64 load_ect_head(void)
{
	return exynos_smc(SMC_CMD_LOAD_ECT_HEAD, 0, 0, 0);
}

u64 load_ect_all(void)
{
	return exynos_smc(SMC_CMD_LOAD_ECT_ALL, 0, 0, 0);
}

u64 skip_ect(void)
{
	return exynos_smc(SMC_CMD_SKIP_ECT, 0, 0, 0);
}
