/* drivers/video/exynos/8890/vpp/vpp_drv.c
 *
 * Samsung EXYNOS5 SoC series VPP driver.
 *
 * Copyright@ Samsung Electronics Co. LTD
 *
 * Manseok Kim <manseoks.kim@samsung.com> *
 * Lee HwangJae <hj-yo.lee@samsung.com> *
 *
 * This software is proprietary of Samsung Electronics.
 * No part of this software, either material or conceptual may be copied or distributed, transmitted,
 * transcribed, stored in a retrieval system or translated into any human or computer language in any form by any means,
 * electronic, mechanical, manual or otherwise, or disclosed
 * to third parties without the express written permission of Samsung Electronics.


 * Alternatively, this program is free software in case of open source projec;
 * you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.

*/


#include <malloc.h>
#include "vpp.h"
#include "vpp_common.h"
#include "../decon_config.h"
#include "../../panels/lcd_ctrl.h"

struct vpp_dev *vpp_for_decon;
EXPORT_SYMBOL(vpp_for_decon);

static void fill_vpp_param(struct decon_win_config *vpp_data)
{
	struct decon_lcd *lcd_info = decon_get_lcd_info();
	vpp_data->src.x = 0;
	vpp_data->src.y = 0;
	vpp_data->src.w = lcd_info->xres;
	vpp_data->src.h = lcd_info->yres;
	vpp_data->dst.w = lcd_info->xres;
	vpp_data->dst.h = lcd_info->yres;
	vpp_data->src.f_w = lcd_info->xres;
	vpp_data->src.f_h = lcd_info->yres;
	vpp_data->format = PIXEL_FORMAT_ARGB_8888;
}

static int vpp_init(struct vpp_dev *vpp)
{
	vpp_reg_init(vpp->id);
	/* CSC_601_NARROW default set */
	vpp_reg_set_rgb_type(vpp->id, 0);

	return 0;
}

int vpp_set_config(u32 dev_id, unsigned long addr)
{
	struct vpp_dev *vpp;
	struct decon_win_config *config;
	struct vpp_size_param p = {0};
	struct vpp_img_format vi = {0};
	int ret = -EINVAL;

	vpp = kzalloc(sizeof(struct vpp_dev), GFP_KERNEL);
	if (!vpp) {
		vpp_err("Failed to allocate local vpp mem\n");
		return -ENOMEM;
	}

	config = kzalloc(sizeof(struct decon_win_config), GFP_KERNEL);
	if (!config) {
		vpp_err("Failed to allocate config mem\n");
		return -ENOMEM;
	}
	fill_vpp_param(config);
	vpp->config = (struct decon_win_config *)config;
	vpp->id = 0x1;
	if(vpp->id == 1)
		vpp->pbuf_num = 0x3;

	vpp_for_decon = vpp;

	vpp->regs = (void __iomem *)VPP_IDMAG1_BASE_ADDR;
	config->vpp_parm.addr = addr;
	vpp_info("%s:VPP%d probe : end\n", __func__, vpp->id);

	ret = vpp_init(vpp);
	if (ret < 0) {
		vpp_err("Failed to initiailze clk\n");
		return ret;
	}

	vpp_reg_wait_pingpong_clear(vpp->id);
	vpp_to_scale_params(vpp, &p);
	ret = vpp_reg_set_in_format(vpp->id, config->format, &vi);
	if (ret)
		goto err;
	vpp_reg_set_in_size(vpp->id, &p);

	config->src.w = p.src_w;
	config->src.h = p.src_h;
	vpp_reg_set_out_size(vpp->id, config->dst.w, config->dst.h);
	vpp_reg_set_in_buf_addr(vpp->id, &p);
	vpp_reg_set_smart_if_pix_num(vpp->id, config->dst.w, config->dst.h);
	vpp_reg_set_in_block_size(vpp->id, false, &p);

	return 0;
err:
	vpp_err("failed to set config\n");
	return ret;
}
