/*
 * copy bitmap to framebuffer
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
*/

#include "asm/types.h"
#include <common.h>
#include <asm/arch/display.h>
#include "./panels/lcd_ctrl.h"

extern uint32_t *win_fb0;

struct bmp_header
{
	// file header
	uint16_t bfType;
	uint32_t bfSize;
	uint32_t bfReserved;
	uint32_t bfOffBits;

	// info header
	uint32_t biSize;
	uint32_t biWidth;
	uint32_t biHeight;
	uint16_t biPlanes;
	uint16_t biBitCount;
	uint32_t biCompression;
	uint32_t biSizeImage;
	uint32_t biXPelsPerMeter;
	uint32_t biYPelsPerMeter;
	uint32_t biClrUsed;
	uint32_t biClrImportant;
} __attribute__ ((packed));

void display_bmp(uint8_t *bmp)
{
	struct bmp_header *bmph = (struct bmp_header *)bmp;
	struct decon_lcd *lcd_info = decon_get_lcd_info();
	int xmargin, ymargin;
	uint32_t pix;
	uint32_t *fbuf;
	int x, y;

	fbuf = win_fb0;

	if (bmph->bfType != 0x4d42 || bmph->biSize != 40 ||
		bmph->biBitCount != 24) {
		return;
	}

	bmp += bmph->bfOffBits;
	bmp += (bmph->biWidth * (bmph->biHeight - 1) * 3);
	xmargin = lcd_info->xres - bmph->biWidth;
	ymargin = lcd_info->yres - bmph->biHeight;
	if (xmargin < 0 || ymargin < 0)
		return;

	fbuf += (ymargin / 2 * lcd_info->xres + xmargin / 2);
	for (y = 0; y < bmph->biHeight; y++) {
		for (x = 0; x < bmph->biWidth; x++) {
			pix = *bmp | *(bmp + 1) << 8 | *(bmp + 2) << 16 | 0xff << 24;
			*fbuf++ = pix;
			bmp += 3;
		}
		bmp -= (bmph->biWidth * 6);
		fbuf += xmargin;
	}
	decon_string_update();
}
